@@ -362,11 +362,7 @@ public class Context extends Entity
         result = CairoContext.getSource(this);
         checkStatus();
 
-        /*
-         * FIXME If result was already proxied, this adds an extra ref and
-         * will cause a memory leak
-         */
-        return CairoPattern.reference(result);
+        return result;
     }
 
     /**
