@@ -1,7 +1,7 @@
 /*
  * GValue.c
  *
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -12,6 +12,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <pango/pango.h>
 #include <jni.h>
 #include "org_gnome_glib_GValue.h"
 #include "bindings_java.h"
@@ -69,21 +70,15 @@ Java_org_gnome_glib_GValue_g_1value_1free
 )
 {
 	GValue* value;
-	GObject* object;
 		
 	value =	(GValue*) _value;
 	
 	/*
-	 * Not 100% sure that this is the correct place to do this, but
-	 * it seems that when you get a property with a GObject in it,
-	 * there is a ref count to that object from the GValue.
+	 * Although this function is mostly about resetting the GValue to
+	 * the blank state, it also has the effect of unref()'ing GObjects,
+	 * free()'ing NULL terminated strings, etc.
 	 */
-	if (G_VALUE_HOLDS_OBJECT(value)) {
-		object = g_value_get_object(value);
-		if (G_IS_OBJECT(object)) {
-			g_object_unref(object);
-		}
-	}
+	g_value_unset(value);
 
 	g_slice_free(GValue, value);
 }
@@ -135,16 +130,16 @@ Java_org_gnome_glib_GValue_g_1value_1init__I
 {
 	gint32 i;
 	GValue* value;
-	
+
 	// translate arg
 	i = (gint32) _i;
-		
+
 	// allocate it and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_INT);
-	
+
 	// set the value
-	g_value_set_int(value, i); 
+	g_value_set_int(value, i);
 
 	// return address
 	return (jlong) value;
@@ -169,16 +164,16 @@ Java_org_gnome_glib_GValue_g_1value_1init__J
 {
 	gint64 j;
 	GValue* value;
-	
+
 	// translate arg
 	j = (gint64) _j;
-		
+
 	// allocate it and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_INT64);
-	
+
 	// set the value
-	g_value_set_int64(value, j); 
+	g_value_set_int64(value, j);
 
 	// return address
 	return (jlong) value;
@@ -206,13 +201,13 @@ Java_org_gnome_glib_GValue_g_1value_1init__Z
 	GValue* value;
 	
 	b = (gboolean) _b;
-		
+
 	// allocate it and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_BOOLEAN);
-	
+
 	// set the value
-	g_value_set_boolean(value, b); 
+	g_value_set_boolean(value, b);
 
 	// return address
 	return (jlong) value;
@@ -243,9 +238,42 @@ Java_org_gnome_glib_GValue_g_1value_1init__F
 	// allocate it and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_FLOAT);
-	
+
 	// set the value
-	g_value_set_float(value, f); 
+	g_value_set_float(value, f);
+
+	// return address
+	return (jlong) value;
+}
+
+/**
+ * Implements
+ *   org.gnome.glib.GValue.g_value_init(double d)
+ * called from
+ *   org.gnome.glib.GValue.createValue(double d)
+ *
+ * Allocate a GValue for a gdouble with GSlice, then initialize it and return
+ * the pointer.
+ */
+JNIEXPORT jlong JNICALL
+Java_org_gnome_glib_GValue_g_1value_1init__D
+(
+	JNIEnv *env,
+	jclass cls,
+	jdouble _d
+)
+{
+	gdouble d;
+	GValue* value;
+	
+	d = (gdouble) _d;
+
+	// allocate it and set to zeros, per what g_value_init requires
+	value =	g_slice_new0(GValue);
+	g_value_init(value, G_TYPE_DOUBLE);
+
+	// set the value
+	g_value_set_double(value, d);
 
 	// return address
 	return (jlong) value;
@@ -270,19 +298,19 @@ Java_org_gnome_glib_GValue_g_1value_1init__Ljava_lang_String_2
 {
 	gchar* str;
 	GValue* value;
-	
+
 	// translate
 	str = (gchar*) (*env)->GetStringUTFChars(env, _str, NULL);
 	if (str == NULL) {
 		return 0; /* OutOfMemoryError already thrown */
 	}
-	
+
 	// allocate and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_STRING);
 
-	// set the value	
-	g_value_set_string(value, str); 
+	// set the value
+	g_value_set_string(value, str);
 
 	// clean up
 	(*env)->ReleaseStringUTFChars(env, _str, str);
@@ -314,12 +342,12 @@ Java_org_gnome_glib_GValue_g_1value_1init_1object
 	
 	// translate obj
 	obj = (GObject*) _obj;
-	
+
 	// allocate and set to zeros, per what g_value_init requires
 	value =	g_slice_new0(GValue);
 	g_value_init(value, G_TYPE_OBJECT);
 
-	// set the value	
+	// set the value
 	g_value_set_object(value, obj);
 
 	// clean up obj
@@ -328,6 +356,130 @@ Java_org_gnome_glib_GValue_g_1value_1init_1object
 	return (jlong) value;
 }
 
+
+/**
+ * Implements
+ *   org.gnome.glib.GValue.g_value_init_font_description(long desc)
+ * called from
+ *   org.gnome.glib.GValue.createValue(FontDescription obj)
+ *
+ * Allocate a GValue for a PangoFontDescription with GSlice, then initialize
+ * it and return the pointer.
+ */
+JNIEXPORT jlong JNICALL
+Java_org_gnome_glib_GValue_g_1value_1init_1font_1description
+(
+    JNIEnv *env,
+    jclass cls,
+    jlong _desc
+)
+{
+    PangoFontDescription* desc;
+    GValue* value;
+    
+    // translate desc
+    desc = (PangoFontDescription*) _desc;
+    
+    // allocate and set to zeros, per what g_value_init requires
+    value = g_slice_new0(GValue);
+    g_value_init(value, PANGO_TYPE_FONT_DESCRIPTION);
+
+    // set the value
+    g_value_set_boxed(value, desc);
+
+    // clean up desc
+
+    // return address
+    return (jlong) value;
+}
+
+
+/**
+ * Implements
+ *   org.gnome.glib.GValue.g_value_init_pixbuf(long pixbuf)
+ * called from
+ *   org.gnome.glib.GValue.createValue(Pixbuf obj)
+ *
+ * Allocate a GValue for a GdkPixbuf with GSlice, then initialize it and return
+ * the pointer.
+ */
+JNIEXPORT jlong JNICALL
+Java_org_gnome_glib_GValue_g_1value_1init_1pixbuf
+(
+    JNIEnv *env,
+    jclass cls,
+    jlong _pixbuf
+)
+{
+    GdkPixbuf* pixbuf;
+    GValue* value;
+    
+    // translate obj
+    pixbuf = (GdkPixbuf*) _pixbuf;
+    
+    // allocate and set to zeros, per what g_value_init requires
+    value = g_slice_new0(GValue);
+    g_value_init(value, GDK_TYPE_PIXBUF);
+
+    // set the value    
+    g_value_set_object(value, pixbuf);
+
+    // clean up obj
+
+    // return address
+    return (jlong) value;
+}
+
+
+/**
+ * Implements
+ *   org.gnome.glib.GValue.g_value_init_enum(int num)
+ * called from
+ *   org.gnome.glib.GValue.createValue(Constant reference)
+ *
+ * Allocate a GValue for a GObject with GSlice, then initialize it and return
+ * the pointer.
+ */
+JNIEXPORT jlong JNICALL
+Java_org_gnome_glib_GValue_g_1value_1init_1enum
+(
+	JNIEnv *env,
+	jclass cls,
+	jstring _name,
+	jint _num
+)
+{
+	gchar* name;
+	GType type;
+	gint num;
+	GValue* value;
+	
+	// translate type;
+	name = (gchar*) (*env)->GetStringUTFChars(env, _name, NULL);
+	if (name == NULL) {
+		return 0; /* OutOfMemoryError already thrown */
+	}
+
+	type = g_type_from_name(name);
+
+	(*env)->ReleaseStringUTFChars(env, _name, name);
+	
+	// translate obj
+	num = (gint) _num;
+	
+	// allocate and set to zeros, per what g_value_init requires
+	value =	g_slice_new0(GValue);
+	g_value_init(value, type);
+
+	// set the value
+	g_value_set_enum(value, num);
+
+	// clean up obj
+
+	// return address
+	return (jlong) value;
+}
+
 /**
  * Implements
  *   org.gnome.glib.GValue.g_value_get_float(long value)
@@ -356,14 +508,50 @@ Java_org_gnome_glib_GValue_g_1value_1get_1float
 		bindings_java_throw(env, "You've asked for the float value of a GValue, but it's not a G_TYPE_FLOAT!");
 		return 0.0f;
 	}
-	
+
 	// call function
 	result = g_value_get_float(value);
-	
+
 	// and return
 	return (jfloat) result; 
 }
 
+/**
+ * Implements
+ *   org.gnome.glib.GValue.g_value_get_double(long value)
+ * called from
+ *   org.gnome.glib.GValue.getDouble(Value value)
+ * called from
+ *   org.gnome.glib.Object.getPropertyDouble(String name)
+ *
+ * Extract the gdoulbe value from a GValue of G_TYPE_DOUBLE, returning the
+ * primitive.
+ */
+JNIEXPORT jdouble JNICALL
+Java_org_gnome_glib_GValue_g_1value_1get_1double
+(
+	JNIEnv* env,
+	jclass cls,
+	jlong _value
+)
+{
+	GValue* value;
+	gdouble result;
+
+	// translate value
+	value =	(GValue*) _value;
+	if (!G_VALUE_HOLDS_DOUBLE(value)) {
+		bindings_java_throw(env, "You've asked for the double value of a GValue, but it's not a G_TYPE_DOUBLE!");
+		return 0.0;
+	}
+
+	// call function
+	result = g_value_get_double(value);
+	
+	// and return
+	return (jdouble) result; 
+}
+
 JNIEXPORT jint JNICALL
 Java_org_gnome_glib_GValue_g_1value_1get_1int
 (
@@ -381,7 +569,7 @@ Java_org_gnome_glib_GValue_g_1value_1get_1int
 		bindings_java_throw(env, "You've asked for the int value of a GValue, but it's not a G_TYPE_INT!");
 		return 0;
 	}
-	
+
 	// call function
 	result = g_value_get_int(value);
 	
@@ -406,10 +594,10 @@ Java_org_gnome_glib_GValue_g_1value_1get_1long
 		bindings_java_throw(env, "You've asked for the long value of a GValue, but it's not a G_TYPE_INT64!");
 		return 0;
 	}
-	
+
 	// call function
 	result = g_value_get_int64(value);
-	
+
 	// and return
 	return (jlong) result;
 }
@@ -432,10 +620,10 @@ Java_org_gnome_glib_GValue_g_1value_1get_1boolean
 		bindings_java_throw(env, "You've asked for the boolean value of a GValue, but it's not a G_TYPE_BOOLEAN!");
 		return 0;
 	}
-	
+
 	// call function
 	result = g_value_get_boolean(value);
-	
+
 	// and return
 	return (jboolean) result; 
 }
@@ -468,9 +656,9 @@ Java_org_gnome_glib_GValue_g_1value_1get_1string
 		bindings_java_throw(env, "You've asked for the string value of a GValue, but it's not a G_TYPE_STRING!");
 		return NULL;
 	}
-	
+
 	// call function
-	str = g_value_get_string(value); 
+	str = g_value_get_string(value);
 
 	// and return	
 	return (*env)->NewStringUTF(env, str);
@@ -503,9 +691,9 @@ Java_org_gnome_glib_GValue_g_1value_1get_1enum
 		bindings_java_throw(env, "You've asked for the ordinal value of a GValue, but it's not a G_TYPE_ENUM!");
 		return 0;
 	}
-	
+
 	// call function
-	num = g_value_get_enum(value); 
+	num = g_value_get_enum(value);
 
 	// and return	
 	return (jint) num;
@@ -538,7 +726,7 @@ Java_org_gnome_glib_GValue_g_1value_1get_1flags
 		bindings_java_throw(env, "You've asked for the flags ordinal value of a GValue, but it's not a G_TYPE_FLAGS!");
 		return 0;
 	}
-	
+
 	// call function
 	num = g_value_get_flags(value); 
 
@@ -570,3 +758,28 @@ Java_org_gnome_glib_GValue_g_1value_1get_1object
 	// and return	
 	return (jlong) object;
 }
+
+JNIEXPORT jlong JNICALL
+Java_org_gnome_glib_GValue_g_1value_1get_1pixbuf
+(
+    JNIEnv* env,
+    jclass cls,
+    jlong _value
+)
+{
+    GValue* value;
+    GdkPixbuf* pixbuf; 
+
+    // translate value
+    value = (GValue*) _value;
+    if (G_VALUE_TYPE(value) != GDK_TYPE_PIXBUF) {
+        bindings_java_throw(env, "You've asked for the GdkPixbuf within a GValue, but it's not a GDK_TYPE_PIXBUF!");
+        return 0L;
+    }
+
+    // call function
+    pixbuf = g_value_get_object(value); 
+
+    // and return   
+    return (jlong) pixbuf;
+}
