@@ -1,8 +1,8 @@
 /*
  * Action.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
- * Copyright (c) 2007 Vreixo Formoso
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007      Vreixo Formoso
  *
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -41,15 +41,15 @@ import org.gnome.glib.Object;
  * properly.
  * 
  * <p>
- * Each Action has an {@link ACTIVATE} signal emitted when any of its proxies
- * are triggered. Thus you can just connect one signal to start the needed
- * operation, instead of having to connect to the <code>ACTIVATE</code>
- * signal of each of various proxies.
+ * Each Action has an {@link Action.Activate} signal emitted when any of its
+ * proxies are triggered. Thus you can just connect one signal to start the
+ * needed operation, instead of having to connect to the
+ * <code>Action.Activate</code> signal of each of various proxies.
  * 
  * <p>
  * Once you have created an Action, you can get proxies for it with the
- * {@link #createMenuItem() createMenuItem()} and
- * {@link #createToolItem() createToolItem()} methods.
+ * {@link #createMenuItem() createMenuItem()} and {@link #createToolItem()
+ * createToolItem()} methods.
  * 
  * <p>
  * Incidentally, you can still use Actions even if you only plan to let the
@@ -69,8 +69,8 @@ public class Action extends Object
     }
 
     /**
-     * Create a new Action, and connect a handler to its <code>ACTIVATE</code>
-     * signal.
+     * Create a new Action, and connect a handler to its
+     * <code>Action.Activate</code> signal.
      * 
      * @param name
      *            A unique name for the Action.
@@ -83,13 +83,20 @@ public class Action extends Object
      * @param stock
      *            The stock icon to display in proxy Widgets.
      * @param handler
-     *            A handler to connect to the <code>ACTIVATE</code> signal.
-     *            Typically this will be used to actually start the operation
-     *            related to this Action.
+     *            A handler to connect to the <code>Action.Activate</code>
+     *            signal. Typically this will be used to actually start the
+     *            operation related to this Action.
+     * @since 4.0.4
      */
     /*
      * FIXME describe the implications of different choices for name.
      */
+    public Action(String name, String label, String tooltip, Stock stock, Action.Activate handler) {
+        super(GtkAction.createAction(name, label, tooltip, stock.getStockId()));
+        connect(handler);
+    }
+
+    /** @deprecated */
     public Action(String name, String label, String tooltip, Stock stock, ACTIVATE handler) {
         super(GtkAction.createAction(name, label, tooltip, stock.getStockId()));
         connect(handler);
@@ -108,6 +115,7 @@ public class Action extends Object
      *            localized.
      * @param stock
      *            The Stock icon to display in proxy Widgets.
+     * @since 4.0.4
      */
     public Action(String name, String label, String tooltip, Stock stock) {
         super(GtkAction.createAction(name, label, tooltip, stock.getStockId()));
@@ -124,8 +132,20 @@ public class Action extends Object
     }
 
     /**
-     * Create a new Action, and connect a handler to its <code>ACTIVATE</code>
-     * signal.
+     * Create a new Action based on a Stock item, and connect a handler to its
+     * <code>Action.Activate</code> signal. Complements the
+     * {@link #Action(String, Stock) &lt;init&gt;(String, Stock)} constructor.
+     * 
+     * @since 4.0.9
+     */
+    public Action(String name, Stock stock, Action.Activate handler) {
+        super(GtkAction.createAction(name, null, null, stock.getStockId()));
+        connect(handler);
+    }
+
+    /**
+     * Create a new Action, and connect a handler to its
+     * <code>Action.Activate</code> signal.
      * 
      * @param name
      *            A unique name for the Action.
@@ -133,10 +153,17 @@ public class Action extends Object
      *            The label that will be displayed in the proxy Widgets. You
      *            usually will want to localize it to the user language.
      * @param handler
-     *            A handler to connect to the <code>ACTIVATE</code> signal.
-     *            Usually will will start from here the operation related to
-     *            the Action.
+     *            A handler to connect to the <code>Action.Activate</code>
+     *            signal. Usually will will start from here the operation
+     *            related to the Action.
+     * @since 4.0.4
      */
+    public Action(String name, String label, Action.Activate handler) {
+        super(GtkAction.createAction(name, label, null, null));
+        connect(handler);
+    }
+
+    /** @deprecated */
     public Action(String name, String label, ACTIVATE handler) {
         super(GtkAction.createAction(name, label, null, null));
         connect(handler);
@@ -150,6 +177,7 @@ public class Action extends Object
      * @param label
      *            The text that will be displayed in the proxy Widgets. You
      *            usually will want to localize it to the user language.
+     * @since 4.0.4
      */
     public Action(String name, String label) {
         super(GtkAction.createAction(name, label, null, null));
@@ -161,8 +189,10 @@ public class Action extends Object
      * <p>
      * You can add the returned MenuItem to a {@link Menu}. The MenuItem will
      * have the same label, icon or accelerator of this Action, and when the
-     * user activates the MenuItem, this Action will <code>ACTIVATE</code>
-     * too.
+     * user activates the MenuItem, this Action will
+     * <code>Action.Activate</code> too.
+     * 
+     * @since 4.0.4
      */
     public MenuItem createMenuItem() {
         return (MenuItem) GtkAction.createMenuItem(this);
@@ -174,7 +204,10 @@ public class Action extends Object
      * <p>
      * You can add the returned ToolItem to a {@link Toolbar}. The ToolItem
      * will have the same Label, icon, tooltips or accelerator of this Action,
-     * and when the user clicks it, this Action will be <code>ACTIVATE</code>D.
+     * and when the user clicks it, this Action will be
+     * <code>Action.Activate</code>d.
+     * 
+     * @since 4.0.4
      */
     public ToolItem createToolItem() {
         return (ToolItem) GtkAction.createToolItem(this);
@@ -205,6 +238,11 @@ public class Action extends Object
      * Finally, note that setting an Action sensitive doesn't always mean it
      * will be actually sensitive, as you also need to make sensitive the
      * ActionGroup the Action belongs to (see {@link #isSensitive()}).
+     * 
+     * <p>
+     * An Action starts life sensitive (ie, <code>true</code>).
+     * 
+     * @since 4.0.4
      */
     public void setSensitive(boolean sensitive) {
         GtkAction.setSensitive(this, sensitive);
@@ -215,6 +253,8 @@ public class Action extends Object
      * user. Note that this doesn't necessarily mean effective sensitivity due
      * to the effect of ActionGroups; see {@link #isSensitive() isSensitive()}
      * for that.
+     * 
+     * @since 4.0.4
      */
     public boolean getSensitive() {
         return GtkAction.getSensitive(this);
@@ -223,6 +263,8 @@ public class Action extends Object
     /**
      * Returns whether the action is effectively sensitive, i.e., if both the
      * Action and the ActionGroup it belongs to are enabled.
+     * 
+     * @since 4.0.4
      */
     public boolean isSensitive() {
         return GtkAction.isSensitive(this);
@@ -236,11 +278,10 @@ public class Action extends Object
      * MenuItems or ToolItems are hidden to the used.
      * 
      * <p>
-     * In most cases, it's a better idea to
-     * {@link #setSensitive(boolean) setSensitive(false)} an Action instead of
-     * make it not visible. That way, users can see that such operation exists
-     * in the application, but they need to do some operations before it
-     * becomes available.
+     * In most cases, it's a better idea to {@link #setSensitive(boolean)
+     * setSensitive(false)} an Action instead of make it not visible. That
+     * way, users can see that such operation exists in the application, but
+     * they need to do some operations before it becomes available.
      * 
      * <p>
      * However, when a full Menu is disabled, this could be a good option. For
@@ -250,7 +291,10 @@ public class Action extends Object
      * <p>
      * Finally, note that setting an Action visible doesn't always mean it
      * will be actually displayed, as you also need to make visible the
-     * ActionGroup the Action belongs to (see {@link #isVisible() isVisible()}).
+     * ActionGroup the Action belongs to (see {@link #isVisible() isVisible()}
+     * for details).
+     * 
+     * @since 4.0.4
      */
     public void setVisible(boolean visible) {
         GtkAction.setVisible(this, visible);
@@ -262,6 +306,7 @@ public class Action extends Object
      * 
      * @see #isVisible()
      * @see #setVisible(boolean)
+     * @since 4.0.4
      */
     public boolean getVisible() {
         return GtkAction.getVisible(this);
@@ -273,6 +318,7 @@ public class Action extends Object
      * @return <code>true</code> when both the Action and the ActionGroup it
      *         belongs to are visible, <code>false</code> otherwise.
      * @see #setVisible(boolean)
+     * @since 4.0.4
      */
     public boolean isVisible() {
         return GtkAction.isVisible(this);
@@ -282,24 +328,34 @@ public class Action extends Object
      * Activates the Action.
      * 
      * <p>
-     * Programmatically cause this Action to fire its <code>ACTIVATE</code>
-     * signal. Note that this has no effect if the Action is not currently
-     * sensitive.
+     * Programmatically cause this Action to fire its
+     * <code>Action.Activate</code> signal. Note that this has no effect if
+     * the Action is not currently sensitive.
      * 
      * <p>
      * Since the Action is automatically activated when user activates one of
      * its proxies (selecting the specific MenuItem or clicking the ToolButton
-     * that goes with this Action), so in most cases <b>you don't need this.</b>
+     * that goes with this Action), normally <b>you don't need this</b>.
      * However, in some cases you want to activate the Action in your
      * application code. Use this there.
+     * 
+     * @since 4.0.8
      */
+    public void emitActivate() {
+        GtkAction.activate(this);
+    }
+
+    /** @deprecated */
     public void activate() {
+        assert false : "use emitActivate() instead";
         GtkAction.activate(this);
     }
 
     /**
      * Set a tooltip (little help message appearing in a hover) for the
      * Action.
+     * 
+     * @since 4.0.4
      */
     public void setTooltip(String tooltip) {
         setPropertyString("tooltip", tooltip);
@@ -307,6 +363,8 @@ public class Action extends Object
 
     /**
      * Get the tooltip for the Action.
+     * 
+     * @since 4.0.4
      */
     public String getTooltip() {
         return getPropertyString("tooltip");
@@ -319,31 +377,45 @@ public class Action extends Object
      * An Action is activated when the user clicks a ToolButton proxy, when
      * (s)he activates an associated MenuItem or when
      * {@link Action#activate() activate()} is called.
+     * 
+     * @since 4.0.4
      */
-    public interface ACTIVATE extends GtkAction.ACTIVATE
+    public interface Activate extends GtkAction.ActivateSignal
     {
         public void onActivate(Action source);
     }
 
     /**
-     * Connect a handler to the <code>ACTIVATE</code> signal.
+     * Connect a handler to the <code>Action.Activate</code> signal.
+     * 
+     * @since 4.0.4
      */
+    public void connect(Action.Activate handler) {
+        GtkAction.connect(this, handler, false);
+    }
+
+    /** @deprecated */
+    public interface ACTIVATE extends GtkAction.ActivateSignal
+    {
+    }
+
+    /** @deprecated */
     public void connect(ACTIVATE handler) {
-        GtkAction.connect(this, handler);
+        assert false : "use Action.Activate instead";
+        GtkAction.connect(this, handler, false);
     }
 
     /**
      * Specify a Widget that will be (another) actor hooked up to this Action.
      * When the proxy Widget is activated (ie if a Button, when it is pressed
      * or clicked , etc) then this Action will be activated and its
-     * <code>ACTIVATE</code> signal will be fired.
+     * <code>Action.Activate</code> signal will be fired.
      * 
      * <p>
      * You use this when you want to use an Action to centralize activity
      * being launched by various different UI controls, but for which the
-     * existing <code>create*</code> proxies are not sufficient. So you
-     * create your Widget separately, then tie it to this Action with this
-     * method.
+     * existing <code>create*</code> proxies are not sufficient. So you create
+     * your Widget separately, then tie it to this Action with this method.
      * 
      * <p>
      * GTK will attempt to "synchronize" the tooltips, labels, and icons in
@@ -355,7 +427,7 @@ public class Action extends Object
      * 
      * nifty = new Action(&quot;nifty&quot;, &quot;Do nifty things!&quot;);
      * nifty.setTooltip(&quot;This will result in amazingly nifty things happening&quot;);
-     * nifty.connect(new Action.ACTIVATE() {
+     * nifty.connect(new Action.Activate() {
      *     public void onActivate(Action source) {
      *     // do something cool
      *     }
@@ -372,10 +444,10 @@ public class Action extends Object
      * nifty.connectProxy(item);
      * </pre>
      * 
-     * will cause the MenuItem's text label to become "<code>Do nifty things!</code>",
-     * and for selecting that MenuItem from the menu to result in the handler
-     * you hooked up to <code>nifty</code>'s <code>ACTIVATE</code> signal
-     * being called.
+     * will cause the MenuItem's text label to become
+     * <code>&quot;Do nifty things!&quot;</code>, and for selecting that
+     * MenuItem from the menu to result in the handler you hooked up to
+     * <code>nifty</code>'s <code>Action.Activate</code> signal being called.
      * 
      * @since 4.0.6
      */
