@@ -181,6 +181,13 @@ public class TreeView extends Container
         return vertical;
     }
 
+    /**
+     * Remove a tree view column from this TreeView.
+     */
+    public void removeColumn(TreeViewColumn column) {
+        GtkTreeView.removeColumn(this, column);
+    }
+    
     /**
      * Set whether this TreeView has a header row at the top of the Widget
      * showing the titles of each of the TreeViewColumns packed into it. The
@@ -193,9 +200,9 @@ public class TreeView extends Container
     /**
      * Set whether the column titles in the header row can be clicked to
      * change the sorting of the displayed data. While the default is
-     * <code>false</code> (since you frequently have the rows ordered the
-     * way they are for a reason and don't want to let the user be reordering
-     * the display and getting lost in the process), calling TreeViewColumn's
+     * <code>false</code> (since you frequently have the rows ordered the way
+     * they are for a reason and don't want to let the user be reordering the
+     * display and getting lost in the process), calling TreeViewColumn's
      * {@link TreeViewColumn#setSortColumn(DataColumn) setSortColumn()} will
      * make the headers clickable. Use this method after your column setup to
      * turn it off [again].
@@ -206,8 +213,9 @@ public class TreeView extends Container
 
     /**
      * Emitted when a row in the TreeView has been activated. Activation
-     * occurs when a row in the view is double-clicked, or when <b><code>Space</code></b>
-     * or <b><code>Enter</code></b> are pressed while a row is selected.
+     * occurs when a row in the view is double-clicked, or when <b>
+     * <code>Space</code></b> or <b><code>Enter</code></b> are pressed while a
+     * row is selected.
      * 
      * <p>
      * In general, you've got the TreeModel and especially its DataColumns
@@ -247,8 +255,8 @@ public class TreeView extends Container
          * converted into a TreeIter with your TreeModel's
          * {@link TreeModel#getIter(TreePath) getIter()} allowing you to then
          * lookup a particular value from the data model. You rarely need
-         * <code>vertical</code> but it can give you some indication in
-         * which column the click happened.
+         * <code>vertical</code> but it can give you some indication in which
+         * column the click happened.
          */
         public void onRowActivated(TreeView source, TreePath path, TreeViewColumn vertical);
     }
@@ -260,6 +268,17 @@ public class TreeView extends Container
         GtkTreeView.connect(this, handler, false);
     }
 
+    /**
+     * Cause a <code>TreeView.RowActivated</code> signal to be emitted for the
+     * given TreePath. The TreeViewColumn is optional; use <code>null</code>
+     * if you don't want to specify it.
+     * 
+     * @since 4.0.9
+     */
+    public void emitRowActivated(TreePath path, TreeViewColumn vertical) {
+        GtkTreeView.rowActivated(this, path, vertical);
+    }
+
     /** @deprecated */
     public interface ROW_ACTIVATED extends GtkTreeView.RowActivatedSignal
     {
@@ -274,10 +293,10 @@ public class TreeView extends Container
     /**
      * Emitted when a row in the TreeView has been expanded, i.e. when its
      * child nodes are shown. A row is expanded either by clicking in the
-     * little arrow near it, or by pressing the <code>+</code> key when a
-     * row is selected. Of course, a row can be only expanded when it has
-     * child rows, and so it can be only emitted when the TreeView is used
-     * with a hierarchical model such as {@link TreeStore}.
+     * little arrow near it, or by pressing the <code>+</code> key when a row
+     * is selected. Of course, a row can be only expanded when it has child
+     * rows, and so it can be only emitted when the TreeView is used with a
+     * hierarchical model such as {@link TreeStore}.
      * 
      * <p>
      * In general, you've got the TreeModel and especially its DataColumns
@@ -368,8 +387,8 @@ public class TreeView extends Container
      * 
      * @param path
      *            The row we want to check.
-     * @return <code>true</code> if the row is expanded, <code>false</code>
-     *         if not.
+     * @return <code>true</code> if the row is expanded, <code>false</code> if
+     *         not.
      * @since 4.0.7
      */
     public boolean rowExpanded(TreePath path) {
@@ -386,8 +405,8 @@ public class TreeView extends Container
      * @param openAll
      *            <code>true</code> to recursively expand all children,
      *            <code>false</code> to expand only the given row.
-     * @return <code>true</code> if the path refers to a valid row, and it
-     *         has child nodes. <code>false</code> otherwise.
+     * @return <code>true</code> if the path refers to a valid row, and it has
+     *         child nodes. <code>false</code> otherwise.
      * @since 4.0.7
      */
     public boolean expandRow(TreePath path, boolean openAll) {
@@ -406,6 +425,8 @@ public class TreeView extends Container
         GtkTreeView.collapseRow(this, path);
     }
 
+    private TreeSelection selection;
+
     /**
      * Get the TreeSelection object corresponding to this TreeView. Every
      * TreeView has a TreeSelection which is a utility instance allowing you
@@ -413,7 +434,10 @@ public class TreeView extends Container
      * method gives you access to it.
      */
     public TreeSelection getSelection() {
-        return GtkTreeView.getSelection(this);
+        if (selection == null) {
+            selection = GtkTreeView.getSelection(this);
+        }
+        return selection;
     }
 
     /**
@@ -557,8 +581,8 @@ public class TreeView extends Container
      * application.
      * 
      * <p>
-     * To fetch the current height mode, see
-     * {@link #getFixedHeightMode() getFixedHeightMode()}.
+     * To fetch the current height mode, see {@link #getFixedHeightMode()
+     * getFixedHeightMode()}.
      * 
      * @param enable
      *            <code>true</code> if all rows in the TreeView are to be of
@@ -635,8 +659,7 @@ public class TreeView extends Container
      * request.
      * 
      * <p>
-     * The default is <code>false</code>, not drawing alternating row
-     * colours.
+     * The default is <code>false</code>, not drawing alternating row colours.
      */
     public void setRulesHint(boolean setting) {
         GtkTreeView.setRulesHint(this, setting);
@@ -661,13 +684,13 @@ public class TreeView extends Container
      * @param rowAlign
      *            Determines where in the view the row specified by
      *            <code>path</code> is placed, with <code>0.0f</code>
-     *            representing top, and <code>1.0f</code> representing
-     *            bottom, as usual. The constants in Alignment such as
+     *            representing top, and <code>1.0f</code> representing bottom,
+     *            as usual. The constants in Alignment such as
      *            {@link Alignment#CENTER CENTER} can be used.
      * @param colAlign
      *            Determines where in the view the column specified by
-     *            <code>vertical</code> will be placed; <code>0.0f</code>
-     *            is fully left, <code>1.0f</code> is fully right.
+     *            <code>vertical</code> will be placed; <code>0.0f</code> is
+     *            fully left, <code>1.0f</code> is fully right.
      * @since 4.0.6
      */
     public void scrollToCell(TreePath path, TreeViewColumn vertical, float rowAlign, float colAlign) {
@@ -676,17 +699,16 @@ public class TreeView extends Container
 
     /**
      * Scroll the TreeView so that the cell specified by <code>path</code>,
-     * <code>vertical</code> is visible. This variant ignores alignment
-     * values and just scrolls the TreeView so that the cell specified is
-     * visible, closest to whichever edge it came in from, and doing nothing
-     * if the cell is already on screen.
+     * <code>vertical</code> is visible. This variant ignores alignment values
+     * and just scrolls the TreeView so that the cell specified is visible,
+     * closest to whichever edge it came in from, and doing nothing if the
+     * cell is already on screen.
      * 
      * <p>
-     * See the discussion about <code>path</code> or <code>vertical</code>
-     * in the other
-     * {@link #scrollToCell(TreePath, TreeViewColumn, float, float) scrollToCell()}
-     * method to learn how you can scroll in a single direction only if
-     * desired.
+     * See the discussion about <code>path</code> or <code>vertical</code> in
+     * the other {@link #scrollToCell(TreePath, TreeViewColumn, float, float)
+     * scrollToCell()} method to learn how you can scroll in a single
+     * direction only if desired.
      * 
      * @since 4.0.6
      */
@@ -705,8 +727,7 @@ public class TreeView extends Container
      * 
      * <p>
      * Incidentally, you can observe these changes by connecting to
-     * <code>TreeView.RowInserted</code> and
-     * <code>TreeView.RowDeleted</code>.
+     * <code>TreeView.RowInserted</code> and <code>TreeView.RowDeleted</code>.
      * 
      * @since 4.0.6
      */
@@ -720,8 +741,8 @@ public class TreeView extends Container
     }
 
     /**
-     * Expand all the rows in this TreeStore backed TreeView, making <i>all</i>
-     * children visible.
+     * Expand all the rows in this TreeStore backed TreeView, making
+     * <i>all</i> children visible.
      * 
      * @since 4.0.7
      */
@@ -739,6 +760,15 @@ public class TreeView extends Container
         GtkTreeView.collapseAll(this);
     }
 
+    /**
+     * Get all the TreeViewColumns for this TreeView.
+     * 
+     * @since 4.0.11
+     */
+    public TreeViewColumn[] getColumns() {
+        return GtkTreeView.getColumns(this);
+    }
+    
     /**
      * Get the TreeViewColumn at the given position in the TreeView, with
      * <code>0</code> being the left-most one.
@@ -764,8 +794,8 @@ public class TreeView extends Container
      * <code>startEditing</code> to <code>true</code> which causes the
      * TreeView to immediately start editing at the the specified row and
      * column (assuming, of course, that that CellRenderer has been made
-     * mutable. See
-     * {@link CellRendererText#setEditable(boolean) setEditable()}).
+     * mutable. See {@link CellRendererText#setEditable(boolean)
+     * setEditable()}).
      * 
      * @since 4.0.8
      */
@@ -773,4 +803,112 @@ public class TreeView extends Container
         GtkTreeView.setCursor(this, path, vertical, startEditing);
     }
 
+    /**
+     * Get a TreePath indicating what row in the TreeView a given set of
+     * co-ordinates corresponds to.
+     * 
+     * <p>
+     * The position indicated by (<code>x</code>,<code>y</code>) is in
+     * <var>bin co-ordinates</var>. Usually you are working in the context of
+     * a handler hooked up to an Event and these values should be obtained
+     * from that Event.
+     * 
+     * <p>
+     * See also {@link #getColumnAtPos(int, int) getColumnAtPos()} for the
+     * complementary method to find out what vertical the co-ordinates
+     * correspond to.
+     * 
+     * <h2>Handling right-clicks</h2>
+     * 
+     * <p>
+     * It is common to create a context menu as a result of a right-click on a
+     * TreeView. Ordinarily, you would intercept the
+     * <code>Widget.ButtonPressEvent</code> signal and then prepare your Menu
+     * in the handler there, quite reasonably expecting that the row that you
+     * have right-clicked on would be selected. Unfortunately, if you hook up
+     * to that signal your code will run before the default handler and the
+     * <i>previously</i> selected row will still be selected while your
+     * handler runs. This is annoying. It is the default
+     * <code>Widget.ButtonPressEvent</code> handler which selects the new row,
+     * so you have to manually select it yourself before acting on the
+     * right-click. It is <code>getPathAtPos()</code> which gives you the
+     * ability to do so:
+     * 
+     * <pre>
+     * view.connect(new Widget.ButtonPressEvent() {
+     *     public boolean onButtonPressEvent(Widget source, EventButton event) {
+     *         final int x, y;
+     *         final TreePath path;
+     *         final TreeSelection selection;
+     * 
+     *         if (event.getButton() != MouseButton.RIGHT) {
+     *             return false;
+     *         }
+     * 
+     *         x = (int) event.getX();
+     *         y = (int) event.getY();
+     *         path = view.getPathAtPos(x, y);
+     * 
+     *         selection = view.getSelection();
+     *         selection.selectRow(path);
+     * 
+     *         // and now pop your context menu, doing
+     *         // something with the row as appropriate.
+     * 
+     *         return true;
+     *     }
+     * });
+     * </pre>
+     * 
+     * @return Will return <code>null</code> if the passed in co-ordinates do
+     *         not correspond to a row in the TreeView.
+     * @since 4.0.9
+     */
+    public TreePath getPathAtPos(int x, int y) {
+        final TreePath[] path;
+        final boolean result;
+
+        path = new TreePath[1];
+
+        result = GtkTreeView.getPathAtPos(this, x, y, path, null, null, null);
+
+        if (result) {
+            return path[0];
+        } else {
+            return null;
+        }
+    }
+
+    /**
+     * Figure out which TreeViewColumn a given event's co-ordinates correspond
+     * to. See {@link #getPathAtPos(int, int) getPathAtPos()} for a detailed
+     * discussion.
+     * 
+     * <p>
+     * <i>In native GTK, this is implemented as an out parameter on the same
+     * function that powers <code>getPathAtPos()</code>, but we've given it a
+     * more coherent name here.</i>
+     * 
+     * @since 4.0.9
+     */
+    /*
+     * This does not match the completion style we have elsewhere in
+     * java-gnome where we use the return to differentiate the various
+     * out-parameters. That is bad, but diverging from getPathAtPos() to
+     * getAtPosPath() seems very distasteful. This is the less used code path.
+     */
+    public TreeViewColumn getColumnAtPos(int x, int y) {
+        final TreeViewColumn[] column;
+        final boolean result;
+
+        column = new TreeViewColumn[1];
+
+        result = GtkTreeView.getPathAtPos(this, x, y, null, column, null, null);
+
+        if (result) {
+            return column[0];
+        } else {
+            return null;
+        }
+    }
 }
