@@ -1,7 +1,7 @@
 /*
  * Window.java
  *
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd, and Others
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -12,10 +12,12 @@
 package org.gnome.gtk;
 
 import org.gnome.gdk.Event;
+import org.gnome.gdk.EventConfigure;
 import org.gnome.gdk.Gravity;
 import org.gnome.gdk.Pixbuf;
 import org.gnome.gdk.Screen;
 import org.gnome.gdk.WindowState;
+import org.gnome.gdk.WindowTypeHint;
 
 /**
  * The top level Widget that contains other Widgets. Typical examples are
@@ -41,7 +43,8 @@ import org.gnome.gdk.WindowState;
  * 
  * <p>
  * <code>hide()</code> is also important when Windows are being closed; see
- * the comments in the description of {@link Window.DELETE_EVENT DELETE_EVENT}.
+ * the comments in the description of {@link Window.DeleteEvent
+ * Window.DeleteEvent}.
  * 
  * @author Andrew Cowie
  * @author Srichand Pendyala
@@ -65,10 +68,9 @@ public class Window extends Bin
 
     /**
      * Create a new Window of the specified type. In general you don't need to
-     * use this; see the comments in
-     * {@link org.gnome.gtk.WindowType WindowType}; in particular,
-     * {@link org.gnome.gtk.WindowType#POPUP POPUP} is <b>not</b> for dialog
-     * windows!
+     * use this; see the comments in {@link org.gnome.gtk.WindowType
+     * WindowType}; in particular, {@link org.gnome.gtk.WindowType#POPUP
+     * POPUP} is <b>not</b> for dialog windows!
      * 
      * @since 4.0.0
      */
@@ -143,8 +145,8 @@ public class Window extends Bin
      * 
      * @since 4.0.0
      */
-    public void setDecorated(boolean decorated) {
-        GtkWindow.setDecorated(this, decorated);
+    public void setDecorated(boolean setting) {
+        GtkWindow.setDecorated(this, setting);
     }
 
     /**
@@ -156,10 +158,15 @@ public class Window extends Bin
      * the size it had prior to hiding, rather than using the default size.
      * 
      * <p>
-     * Depending on your needs, {@link #resize() resize()} could be more
-     * appropriate, especially if the Window is already realized . resize()
-     * changes the current size of the Window, rather than the size to be used
-     * on initial display which is what this method is for.
+     * Depending on your needs, {@link #resize(int, int) resize()} could be
+     * more appropriate, especially if the Window is already realized.
+     * <code>resize()</code> changes the current size of the Window, rather
+     * than the size to be used on initial display which is what this method
+     * is for.
+     * 
+     * <p>
+     * You probably want to call this after you've done the bulk of your
+     * packing.
      * 
      * <p>
      * Incidentally, Windows can't be 0x0; the minimum size is 1x1.
@@ -183,9 +190,8 @@ public class Window extends Bin
      * 
      * <p>
      * Somewhat unusually, if the new value for <code>position</code> is
-     * {@link WindowPosition#CENTER_ALWAYS CENTER_ALWAYS}, then this call
-     * will also result in the Window being moved to the new centered
-     * position.
+     * {@link WindowPosition#CENTER_ALWAYS CENTER_ALWAYS}, then this call will
+     * also result in the Window being moved to the new centered position.
      * 
      * @since 4.0.3
      * @see WindowPosition
@@ -207,8 +213,8 @@ public class Window extends Bin
      * 
      * @param setting
      *            <code>true</code> to request fullscreen mode,
-     *            <code>false</code> to request that the window be returned
-     *            to normal management.
+     *            <code>false</code> to request that the window be returned to
+     *            normal management.
      * @since 4.0.4
      */
     public void setFullscreen(boolean setting) {
@@ -226,8 +232,8 @@ public class Window extends Bin
      * 
      * <p>
      * If you want to prevent a Window from being closed, connect this signal,
-     * and return <code>true</code>. Often the reason to do this is to pop
-     * up a notification Dialog, for example asking you if you want to save an
+     * and return <code>true</code>. Often the reason to do this is to pop up
+     * a notification Dialog, for example asking you if you want to save an
      * unsaved document. Another technique is reusing a Window: rather than
      * going to all the trouble to create this Window again, you can just
      * temporarily hide it by calling Widget's {@link Widget#hide() hide()}.
@@ -252,13 +258,28 @@ public class Window extends Bin
      * @author Devdas Bhagat
      * @since 4.0.0
      */
-    public interface DELETE_EVENT extends GtkWidget.DELETE_EVENT
+    public interface DeleteEvent extends GtkWidget.DeleteEventSignal
     {
         public boolean onDeleteEvent(Widget source, Event event);
     }
 
+    public void connect(DeleteEvent handler) {
+        GtkWidget.connect(this, handler, false);
+    }
+
+    /**
+     * @deprecated
+     */
+    public interface DELETE_EVENT extends GtkWidget.DeleteEventSignal
+    {
+    }
+
+    /**
+     * @deprecated
+     */
     public void connect(DELETE_EVENT handler) {
-        GtkWidget.connect(this, handler);
+        assert false : "use Window.DeleteEvent instead";
+        GtkWidget.connect(this, handler, false);
     }
 
     /**
@@ -272,16 +293,17 @@ public class Window extends Bin
      * 
      * <p>
      * Chances are
-     * {@link org.gnome.gtk.Window#setPosition(org.gnome.gtk.WindowPosition) setPosition()}
-     * will do what you want more easily than manually moving the Window.
+     * {@link org.gnome.gtk.Window#setPosition(org.gnome.gtk.WindowPosition)
+     * setPosition()} will do what you want more easily than manually moving
+     * the Window.
      * 
      * <p>
-     * <i>The co-ordinates <code>x</code>, <code>y</code> are with
-     * respect to the reference point specified by the "gravity" in effect for
-     * this Window; since the default is</i>
-     * {@link Gravity#NORTH_WEST NORTH_WEST}<i>, x and y mean what you want
-     * them to: horizontal and vertical distance of the top-left corner of the
-     * Window from the top-left corner, respectively.</i>
+     * <i>The co-ordinates <code>x</code>, <code>y</code> are with respect to
+     * the reference point specified by the "gravity" in effect for this
+     * Window; since the default is</i> {@link Gravity#NORTH_WEST NORTH_WEST}
+     * <i>, x and y mean what you want them to: horizontal and vertical
+     * distance of the top-left corner of the Window from the top-left corner,
+     * respectively.</i>
      * 
      * @since 4.0.4
      */
@@ -291,8 +313,8 @@ public class Window extends Bin
 
     /**
      * Set the interpretation of co-ordinates passed to
-     * {@link #move(int, int)} and returned by
-     * {@link #getPositionX() getPosition()}.
+     * {@link #move(int, int)} and returned by {@link #getPositionX()
+     * getPosition()}.
      * 
      * <p>
      * <i>The window manager specification has long been notorious for not
@@ -403,20 +425,20 @@ public class Window extends Bin
      * desktop.
      * 
      * <p>
-     * You can call this before <code>show()</code>ing a Window, in which
-     * case the initial presentation will be behind other windows.
+     * You can call this before <code>show()</code>ing a Window, in which case
+     * the initial presentation will be behind other windows.
      * 
      * <p>
-     * The caveats and notes discussed in
-     * {@link #setKeepAbove(boolean) setKeepAbove()} apply here. Once again,
-     * while there are legitimate uses for this method, please think about the
-     * impact on user's overall desktop experience before employing it.
+     * The caveats and notes discussed in {@link #setKeepAbove(boolean)
+     * setKeepAbove()} apply here. Once again, while there are legitimate uses
+     * for this method, please think about the impact on user's overall
+     * desktop experience before employing it.
      * 
      * 
      * @param setting
-     *            <code>true</code> to request this Window be kept behind
-     *            all other windows on the desktop, <code>false</code> for
-     *            normal behaviour.
+     *            <code>true</code> to request this Window be kept behind all
+     *            other windows on the desktop, <code>false</code> for normal
+     *            behaviour.
      * @since 4.0.4
      */
     public void setKeepBelow(boolean setting) {
@@ -448,8 +470,8 @@ public class Window extends Bin
      * 
      * @param setting
      *            <code>true</code> to request the Window be stuck,
-     *            <code>false</code> to request a return to the normal
-     *            default state.
+     *            <code>false</code> to request a return to the normal default
+     *            state.
      * @since 4.0.4
      */
     public void setStick(boolean setting) {
@@ -465,13 +487,13 @@ public class Window extends Bin
      * restore (return to normal) this Window.
      * 
      * <p>
-     * The difference between this and
-     * {@link #setFullscreen(boolean) setFullscreen(true)} is that a maximized
-     * Window still has the title at the top of the screen (along with other
-     * window decorations, depending on the theme), and the panel(s) are still
-     * visible; a fullscreen window is over top of <i>everything</i>. While
-     * there are legitimate uses for both, maximizing is somewhat preferable
-     * since it does not obscure the panels.
+     * The difference between this and {@link #setFullscreen(boolean)
+     * setFullscreen(true)} is that a maximized Window still has the title at
+     * the top of the screen (along with other window decorations, depending
+     * on the theme), and the panel(s) are still visible; a fullscreen window
+     * is over top of <i>everything</i>. While there are legitimate uses for
+     * both, maximizing is somewhat preferable since it does not obscure the
+     * panels.
      * 
      * <p>
      * The default for new Windows in ordinary circumstances is that they are
@@ -508,9 +530,9 @@ public class Window extends Bin
      * <p>
      * While this is in the form of a getter, <var>maximize</var> is not a
      * property, as such. This method only works (ie, reports
-     * <code>true</code>) if the Window is actually on the screen (ie has
-     * been mapped via a <code>show()</code> call), and really <b>is</b>
-     * currently maximized.
+     * <code>true</code>) if the Window is actually on the screen (ie has been
+     * mapped via a <code>show()</code> call), and really <b>is</b> currently
+     * maximized.
      * 
      * <p>
      * In other words, this won't do you any good when initializing your
@@ -568,15 +590,15 @@ public class Window extends Bin
      * Present the Window to the user. This will raise the window to the top
      * of the stack, deiconify it, and even bring it to the current virtual
      * workspace (all depending, as ever, on how co-operative your window
-     * manager is). This is also invokes the equivalent of
-     * {@link #show() show()} to [re]map the Window.
+     * manager is). This is also invokes the equivalent of {@link #show()
+     * show()} to [re]map the Window.
      * 
      * <p>
      * This method is ideal for cases where a Window is already showing
      * somewhere and you need to [re]present it to the user. It's also what
-     * you should use if you have already called <code>show()</code> to
-     * force the Window to map but then immediately called <code>hide()</code>
-     * while you finished building the Window.
+     * you should use if you have already called <code>show()</code> to force
+     * the Window to map but then immediately called <code>hide()</code> while
+     * you finished building the Window.
      * 
      * @since 4.0.5
      */
@@ -601,12 +623,12 @@ public class Window extends Bin
      * accessibility and user preference. Also, GTK is unable to take into
      * account the size of any window decorations that may be present. Use
      * {@link #setPosition(WindowPosition) setPosition()}!
-     * <li>If you need to take a dynamic size dependent action you should
-     * hook up to the {@link Widget.CONFIGURE_EVENT CONFIGURE_EVENT} signal
-     * which has more accurate information and which will allow you to react
-     * appropriately. If you instead use this you will be subject to a race
-     * condition as the size of the Window may change between you calling this
-     * method and taking action based on the returned value.
+     * <li>If you need to take a dynamic size dependent action you should hook
+     * up to the {@link Window.ConfigureEvent} signal which has more accurate
+     * information and which will allow you to react appropriately. If you
+     * instead use this you will be subject to a race condition as the size of
+     * the Window may change between you calling this method and taking action
+     * based on the returned value.
      * </ul>
      * 
      * In other words, although this method can be useful for debugging, it's
@@ -696,4 +718,152 @@ public class Window extends Bin
     public void setTransientFor(Window parent) {
         GtkWindow.setTransientFor(this, parent);
     }
+
+    /**
+     * Set whether the user can resize this Window. Windows are realizable by
+     * default.
+     * 
+     * <p>
+     * You'd probably best have a very good reason for suppressing this. GTK's
+     * box packing model and size request/size allocation cycle will draw a
+     * Window at the optimum size necessary for all the Widgets packed into it
+     * to fit and so that all text labels draw properly, etc. That said,
+     * sometimes a user wants to see more of a certain column or to expand the
+     * area given to the Window so that some dynamic element within (perhaps
+     * an image) will scale larger. It is very frustrating for a user to find
+     * that the developer has prohibited them from doing such things.
+     * 
+     * @since 4.0.6
+     */
+    public void setResizable(boolean setting) {
+        GtkWindow.setResizable(this, setting);
+    }
+
+    /**
+     * Resize the Window to the given <code>width</code> and
+     * <code>height</code>.
+     * 
+     * <p>
+     * The size specified here needs to meet or exceed the current aggregate
+     * size request for this Window as determined by the children packed into
+     * it. If that's a problem, then influence the size request with
+     * {@link #setSizeRequest(int, int) setSizeRequest()}.
+     * 
+     * <p>
+     * If you call this before the Window is realized, then the settings
+     * provided here will override those specified with
+     * {@link #setDefaultSize(int, int) setDefaultSize()}.
+     * 
+     * @since 4.0.8
+     */
+    public void resize(int width, int height) {
+        if ((width < 1) || (height < 1)) {
+            throw new IllegalArgumentException("absolute minimum size is 1x1");
+        }
+        GtkWindow.resize(this, width, height);
+    }
+
+    /**
+     * Request that GNOME not include this Window in lists of open windows.
+     * These are notably the "Window List" and "Window Selector" applets
+     * included with <code>gnome-panel</code> and the on-screen-display popup
+     * presented by the window manager. This is useful when creating special
+     * purpose auxiliary windows that are not the main program.
+     * 
+     * <p>
+     * Like other "hint" setting methods, this is only a request to the
+     * external environment and could potentially be ignored.
+     * 
+     * <p>
+     * You may also need {@link #setSkipPagerHint(boolean) setSkipPagerHint()}.
+     * 
+     * <p>
+     * Note that if the WindowTypeHint of a Window has been set appropriately,
+     * you will not need to call this. Therefore use
+     * {@link #setTypeHint(WindowTypeHint) setTypeHint()} instead.
+     * 
+     * @since 4.0.8
+     */
+    public void setSkipTaskbarHint(boolean setting) {
+        GtkWindow.setSkipTaskbarHint(this, setting);
+    }
+
+    /**
+     * Request that GNOME not include this Window in pagers. Pagers are
+     * applets and other utilities which show thumbnails of windows as an aide
+     * to navigation and switching. The "Workspace Switcher" applet included
+     * with <code>gnome-panel</code> is a pager.
+     * 
+     * <p>
+     * If you're trying to keep this Window off of the list of windows shown
+     * as buttons in a panel, then you're probably looking for
+     * {@link #setSkipTaskbarHint(boolean) setSkipTaskbarHint()} instead,
+     * although this can be a nice touch too.
+     * 
+     * <p>
+     * Note that if the WindowTypeHint of a Window has been set appropriately,
+     * you will not need to call this. Therefore use
+     * {@link #setTypeHint(WindowTypeHint) setTypeHint()} instead.
+     * 
+     * @since 4.0.8
+     */
+    public void setSkipPagerHint(boolean setting) {
+        GtkWindow.setSkipPagerHint(this, setting);
+    }
+
+    /**
+     * Indicate to the window manager what type of use this Window will be put
+     * to. While the default is {@link WindowTypeHint#NORMAL NORMAL}, you may
+     * find the greatest utility from calling this with the
+     * {@link WindowTypeHint#UTILITY UTILITY} hint.
+     * 
+     * @since 4.0.8
+     */
+    /*
+     * Yes, that's a bad pun. I dare you to do better!
+     */
+    public void setTypeHint(WindowTypeHint hint) {
+        GtkWindow.setTypeHint(this, hint);
+    }
+
+    /**
+     * Event emitted when the Window's size or position changes. The
+     * {@link EventConfigure EventConfigure} object has the position and size
+     * information.
+     * 
+     * <p>
+     * This event will also be emitted when the Window is first mapped and
+     * when it reappears on the screen having been obscured, so don't count on
+     * the values received being different from a previous iteration.
+     * 
+     * <p>
+     * <i>Note that this event signal plays a fairly crucial role in GTK
+     * internally; it is used by numerous subsystems (notably the size-request
+     * / size-allocation mechanism) to propagate that a Window had a new
+     * configuration. Do not attempt to block this signal.</i>
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.8
+     */
+    /*
+     * This is here for the same reason that DeleteEvent is.
+     */
+    public interface ConfigureEvent extends GtkWidget.ConfigureEventSignal
+    {
+        /**
+         * Return <code>false</code>! Although this is an event signal with a
+         * boolean return, there is no point in attempting to block further
+         * propagation.
+         */
+        public boolean onConfigureEvent(Widget source, EventConfigure event);
+    }
+
+    /**
+     * Hook up a <code>Window.ConfigureEvent</code> handler.
+     * 
+     * @since 4.0.8
+     */
+    public void connect(ConfigureEvent handler) {
+        GtkWidget.connect(this, handler, false);
+    }
 }
