@@ -1,9 +1,9 @@
 /*
  * UnitTests.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the suite it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -14,17 +14,40 @@ import junit.framework.TestSuite;
 
 import org.freedesktop.bindings.Debug;
 import org.freedesktop.bindings.ValidateEnvironment;
+import org.freedesktop.bindings.ValidateInternationalization;
+import org.freedesktop.cairo.ValidateCairoInternals;
+import org.freedesktop.cairo.ValidateDrawingToFile;
+import org.gnome.gdk.ValidateImageHandling;
+import org.gnome.gdk.ValidateKeyboardHandling;
 import org.gnome.gdk.ValidateScreensAndDisplays;
 import org.gnome.glib.ValidateConstants;
 import org.gnome.glib.ValidateGListMethods;
 import org.gnome.glib.ValidateMemoryManagement;
 import org.gnome.glib.ValidateReferenceCounting;
 import org.gnome.gtk.TestCaseGtk;
+import org.gnome.gtk.ValidateArrow;
+import org.gnome.gtk.ValidateAssistant;
+import org.gnome.gtk.ValidateComboBox;
 import org.gnome.gtk.ValidateFileChoosing;
+import org.gnome.gtk.ValidateIconView;
+import org.gnome.gtk.ValidateNotebookBehaviour;
 import org.gnome.gtk.ValidateOutParameters;
 import org.gnome.gtk.ValidatePacking;
+import org.gnome.gtk.ValidatePrinting;
 import org.gnome.gtk.ValidateProperties;
+import org.gnome.gtk.ValidateResponseType;
+import org.gnome.gtk.ValidateSignalEmission;
+import org.gnome.gtk.ValidateSnapshotUtilities;
 import org.gnome.gtk.ValidateStockItems;
+import org.gnome.gtk.ValidateTextBuffer;
+import org.gnome.gtk.ValidateTextViewBorderWindows;
+import org.gnome.gtk.ValidateTextViewProperties;
+import org.gnome.gtk.ValidateTreeModel;
+import org.gnome.gtk.ValidateTreeModelFilter;
+import org.gnome.gtk.ValidateTreeStore;
+import org.gnome.gtk.ValidateTreeView;
+import org.gnome.pango.ValidatePangoAttributeUsage;
+import org.gnome.pango.ValidatePangoTextRendering;
 
 import com.operationaldynamics.codegen.ValidateThingUsage;
 import com.operationaldynamics.codegen.ValidateUtilityMethods;
@@ -97,16 +120,39 @@ public class UnitTests
         suite.addTestSuite(ValidateDefsParsing.class);
         suite.addTestSuite(ValidateThingUsage.class);
         suite.addTestSuite(ValidateEnvironment.class);
+        suite.addTestSuite(ValidateInternationalization.class);
         suite.addTestSuite(ValidateReferenceCounting.class);
         suite.addTestSuite(ValidateMemoryManagement.class);
         suite.addTestSuite(ValidateGListMethods.class);
         suite.addTestSuite(ValidateConstants.class);
         suite.addTestSuite(ValidateProperties.class);
+        suite.addTestSuite(ValidateSignalEmission.class);
         suite.addTestSuite(ValidateScreensAndDisplays.class);
+        suite.addTestSuite(ValidateKeyboardHandling.class);
+        suite.addTestSuite(ValidateImageHandling.class);
+        suite.addTestSuite(ValidateCairoInternals.class);
+        suite.addTestSuite(ValidateDrawingToFile.class);
         suite.addTestSuite(ValidateOutParameters.class);
         suite.addTestSuite(ValidatePacking.class);
+        suite.addTestSuite(ValidateNotebookBehaviour.class);
         suite.addTestSuite(ValidateFileChoosing.class);
         suite.addTestSuite(ValidateStockItems.class);
+        suite.addTestSuite(ValidateResponseType.class);
+        suite.addTestSuite(ValidateTreeModel.class);
+        suite.addTestSuite(ValidateTreeStore.class);
+        suite.addTestSuite(ValidateTreeModelFilter.class);
+        suite.addTestSuite(ValidateTreeView.class);
+        suite.addTestSuite(ValidateIconView.class);
+        suite.addTestSuite(ValidateComboBox.class);
+        suite.addTestSuite(ValidateSnapshotUtilities.class);
+        suite.addTestSuite(ValidateAssistant.class);
+        suite.addTestSuite(ValidateTextBuffer.class);
+        suite.addTestSuite(ValidateTextViewProperties.class);
+        suite.addTestSuite(ValidateTextViewBorderWindows.class);
+        suite.addTestSuite(ValidateArrow.class);
+        suite.addTestSuite(ValidatePangoTextRendering.class);
+        suite.addTestSuite(ValidatePangoAttributeUsage.class);
+        suite.addTestSuite(ValidatePrinting.class);
 
         return suite;
     }
